/////////////////////////////////////////////////////////////////////////////////
// Company: 武汉芯路恒科技有限公司
// Engineer: 小梅哥团队
// Web: www.corecourse.cn
// 
// Create Date: 2019/05/01 00:00:00
// Design Name: 
// Module Name: udp_gmii_ddr3_tft
// Project Name: 
// Target Devices: XC7A35T-2FGG484I
// Tool Versions: Vivado 2018.3
// Description: 以太网传图，DDR3缓存，TFT显示，默认5寸TFT屏
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

`include "../define/define.v"

module udp_gmii_ddr3_tft(
  //System clock reset
  input           sys_clk       , //系统时钟输入，50M
  input           sys_rst_n     , //复位信号输入，低有效
  //LED
  output [1:0]    led           ,
  //eth interface              
  (* clock_buffer_type="NONE" *)input  rgmii_rx_clk,
	input  [3:0]    rgmii_rxd     ,
	input           rgmii_rxdv    ,
  output          eth_reset_n   ,
  //TFT Interface               
	output [15:0]   TFT_rgb       , //TFT数据输出
	output          TFT_hs        , //TFT行同步信号
	output          TFT_vs        , //TFT场同步信号
	output          TFT_clk       , //TFT像素时钟
	output          TFT_de        , //TFT数据使能
	output          TFT_pwm       , //TFT背光控制
  //DDR3 Interface
  // Inouts
  inout  [15:0]   ddr3_dq       ,
  inout  [1:0]    ddr3_dqs_n    ,
  inout  [1:0]    ddr3_dqs_p    ,
  // Outputs      
  output [13:0]   ddr3_addr     ,
  output [2:0]    ddr3_ba       ,
  output          ddr3_ras_n    ,
  output          ddr3_cas_n    ,
  output          ddr3_we_n     ,
  output          ddr3_reset_n  ,
  output [0:0]    ddr3_ck_p     ,
  output [0:0]    ddr3_ck_n     ,
  output [0:0]    ddr3_cke      ,
  output [0:0]    ddr3_cs_n     ,
  output [1:0]    ddr3_dm       ,
  output [0:0]    ddr3_odt      
);
//*********************************
//define
//*********************************
`ifdef SEL_TFT_4_3
  parameter IMAGE_WIDTH  = 480;
  parameter IMAGE_HEIGHT = 272;
  parameter PIXEL_BYTE   = 2  ;
`else
  parameter IMAGE_WIDTH  = 800;
  parameter IMAGE_HEIGHT = 480;
  parameter PIXEL_BYTE   = 2  ;
`endif

//*********************************
//Parameter
//*********************************
  localparam LOCAL_MAC  = 48'h00_0a_35_01_fe_c0;
  localparam LOCAL_IP   = 32'hc0_a8_00_02;
  localparam LOCAL_PORT = 16'd5000;

//*********************************
//Internal connect
//*********************************
  //clock
  wire          pll_locked;
  wire          tft_ctrl_clk33M;
  wire          tft_ctrl_clk9M;
  wire          mig_clk200M;
  //reset
  wire          g_rst_n;
  wire          mig_reset_n;
  //eth Interface
  wire [3 :0]   rgmii_rxd_idly;
  wire          rgmii_rxdv_idly;
  wire          fifo_full;
  wire          fifo_wr;
  wire [7 :0]   fifo_din;
  wire          eth_rx_clk;
  wire          one_pkt_done;
  wire          pkt_error;
  //wr_fifo Interface
  wire [15:0]   wrfifo_din;
  wire          wrfifo_wren;
  wire          wrfifo_rden;
  wire [127:0]  wrfifo_dout;
  wire [5 : 0]  wrfifo_rd_cnt;
  wire          wrfifo_empty;
  wire          wrfifo_wr_rst_busy;
  wire          wrfifo_rd_rst_busy;
  //rd_fifo Interface
  wire          rdfifo_wren;
  wire [127:0]  rdfifo_din;
  wire          rdfifo_rden;
  wire [15 :0]  rdfifo_dout;
  wire [5 : 0]  rdfifo_wr_cnt;
  wire          rdfifo_full;
  wire          rdfifo_wr_rst_busy;
  wire          rdfifo_rd_rst_busy;
  //mig Interface
  wire			    ui_clk;
  wire			    ui_clk_sync_rst;
  wire			    mmcm_locked;
  wire			    init_calib_complete;
  wire		      aresetn;

  wire[3:0]			s_axi_awid;
  wire[27:0]	  s_axi_awaddr;
  wire[7:0]			s_axi_awlen;
  wire[2:0]			s_axi_awsize;
  wire[1:0]			s_axi_awburst;
  wire[0:0]			s_axi_awlock;
  wire[3:0]			s_axi_awcache;
  wire[2:0]			s_axi_awprot;
  wire[3:0]			s_axi_awqos;
  wire		      s_axi_awvalid;
  wire			    s_axi_awready;

  wire[127:0]	  s_axi_wdata;
  wire[15:0]	  s_axi_wstrb;
  wire		      s_axi_wlast;
  wire		      s_axi_wvalid;
  wire			    s_axi_wready;

  wire [3:0]	  s_axi_bid;
  wire [1:0]	  s_axi_bresp;
  wire			    s_axi_bvalid;
  wire		      s_axi_bready;

  wire[3:0]			s_axi_arid;
  wire[27:0]	  s_axi_araddr;
  wire[7:0]			s_axi_arlen;
  wire[2:0]			s_axi_arsize;
  wire[1:0]			s_axi_arburst;
  wire[0:0]			s_axi_arlock;
  wire[3:0]			s_axi_arcache;
  wire[2:0]			s_axi_arprot;
  wire[3:0]			s_axi_arqos;
  wire		      s_axi_arvalid;
  wire			    s_axi_arready;

  wire [3:0]	  s_axi_rid;
  wire [127:0]  s_axi_rdata;
  wire [1:0]	  s_axi_rresp;
  wire			    s_axi_rlast;
  wire			    s_axi_rvalid;
  wire		      s_axi_rready;
  
  wire          tft_ctrl_clk;
  wire          frame_begin;

  wire          rdfifo_clr;  
  reg           rdfifo_clr_sync_ui_clk;
  reg           rd_addr_clr;  
  wire          wrfifo_clr;
  reg           wrfifo_clr_sync_ui_clk;
  reg           wr_addr_clr;
  reg           wr_fifo_rst_busy_pre;
  reg           wr_fifo_rst_busy;
  reg           rd_fifo_rst_busy_pre;
  reg           rd_fifo_rst_busy;
  
`ifdef SEL_TFT_4_3
  assign tft_ctrl_clk = tft_ctrl_clk9M;
`else
  assign tft_ctrl_clk = tft_ctrl_clk33M;
`endif
  
  assign aresetn     = pll_locked ;//&& rx_clk_locked;
  assign mig_reset_n = pll_locked ;//&& rx_clk_locked;
  assign g_rst_n     = ~ui_clk_sync_rst;
  assign eth_reset_n = 1'b1;
  assign led         = {init_calib_complete,mmcm_locked};

  pll pll
  (
    // Clock out ports
    .clk_out1 (mig_clk200M     ), // output clk_out1
    .clk_out2 (tft_ctrl_clk33M ), // output clk_out2
    .clk_out3 (tft_ctrl_clk9M  ), // output clk_out3
    // Status and control signals
    .reset    (~sys_rst_n      ), // input reset
    .locked   (pll_locked      ), // output locked
    // Clock in ports
    .clk_in1  (sys_clk         )  // input clk_in1
  );

//---------------------------------------------------
//下面部分针对RGMII输入延时不能满足时序要求，对数据线
//和控制线使用IDELAY先进行一定延时满足要求后在进入IDDR
//---------------------------------------------------
  IDELAYCTRL IDELAYCTRL_inst (
    .RDY   (                ), // 1-bit output: Ready output
    .REFCLK(mig_clk200M     ), // 1-bit input: Reference clock input
    .RST   (ui_clk_sync_rst )  // 1-bit input: Active high reset input
  );

  genvar i;
  generate
    for(i=0;i<4;i=i+1) 
    begin: rgmii_rxd_dly
    
      IDELAYE2 #(
        .CINVCTRL_SEL("FALSE"),          // Enable dynamic clock inversion (FALSE, TRUE)
        .DELAY_SRC("IDATAIN"),           // Delay input (IDATAIN, DATAIN)
        .HIGH_PERFORMANCE_MODE("FALSE"), // Reduced jitter ("TRUE"), Reduced power ("FALSE")
        .IDELAY_TYPE("FIXED"),           // FIXED, VARIABLE, VAR_LOAD, VAR_LOAD_PIPE
        .IDELAY_VALUE(5),                // Input delay tap setting (0-31)
        .PIPE_SEL("FALSE"),              // Select pipelined mode, FALSE, TRUE
        .REFCLK_FREQUENCY(200.0),        // IDELAYCTRL clock input frequency in MHz (190.0-210.0, 290.0-310.0).
        .SIGNAL_PATTERN("DATA")          // DATA, CLOCK input signal
      )
      IDELAYE2_inst (
        .CNTVALUEOUT(                 ), // 5-bit output: Counter value output
        .DATAOUT    (rgmii_rxd_idly[i]), // 1-bit output: Delayed data output
        .C          (1'b0             ), // 1-bit input: Clock input
        .CE         (1'b0             ), // 1-bit input: Active high enable increment/decrement input
        .CINVCTRL   (1'b0             ), // 1-bit input: Dynamic clock inversion input
        .CNTVALUEIN (5'd0             ), // 5-bit input: Counter value input
        .DATAIN     (1'b0             ), // 1-bit input: Internal delay data input
        .IDATAIN    (rgmii_rxd[i]     ), // 1-bit input: Data input from the I/O
        .INC        (1'b0             ), // 1-bit input: Increment / Decrement tap delay input
        .LD         (1'b0             ), // 1-bit input: Load IDELAY_VALUE input
        .LDPIPEEN   (1'b0             ), // 1-bit input: Enable PIPELINE register to load data input
        .REGRST     (ui_clk_sync_rst  )  // 1-bit input: Active-high reset tap-delay input
      );
    end
  endgenerate

  IDELAYE2 #(
    .CINVCTRL_SEL("FALSE"),          // Enable dynamic clock inversion (FALSE, TRUE)
    .DELAY_SRC("IDATAIN"),           // Delay input (IDATAIN, DATAIN)
    .HIGH_PERFORMANCE_MODE("FALSE"), // Reduced jitter ("TRUE"), Reduced power ("FALSE")
    .IDELAY_TYPE("FIXED"),           // FIXED, VARIABLE, VAR_LOAD, VAR_LOAD_PIPE
    .IDELAY_VALUE(5),                // Input delay tap setting (0-31)
    .PIPE_SEL("FALSE"),              // Select pipelined mode, FALSE, TRUE
    .REFCLK_FREQUENCY(200.0),        // IDELAYCTRL clock input frequency in MHz (190.0-210.0, 290.0-310.0).
    .SIGNAL_PATTERN("DATA")          // DATA, CLOCK input signal
  )
  IDELAYE2_rxdv (
    .CNTVALUEOUT(                 ), // 5-bit output: Counter value output
    .DATAOUT    (rgmii_rxdv_idly  ), // 1-bit output: Delayed data output
    .C          (1'b0             ), // 1-bit input: Clock input
    .CE         (1'b0             ), // 1-bit input: Active high enable increment/decrement input
    .CINVCTRL   (1'b0             ), // 1-bit input: Dynamic clock inversion input
    .CNTVALUEIN (5'd0             ), // 5-bit input: Counter value input
    .DATAIN     (1'b0             ), // 1-bit input: Internal delay data input
    .IDATAIN    (rgmii_rxdv       ), // 1-bit input: Data input from the I/O
    .INC        (1'b0             ), // 1-bit input: Increment / Decrement tap delay input
    .LD         (1'b0             ), // 1-bit input: Load IDELAY_VALUE input
    .LDPIPEEN   (1'b0             ), // 1-bit input: Enable PIPELINE register to load data input
    .REGRST     (ui_clk_sync_rst  )  // 1-bit input: Active-high reset tap-delay input
  );
//---------------------------------------------------

  udp_rgmii_rx udp_rgmii_rx(
    .reset_n         (g_rst_n               ),

    .rgmii_rx_en     (1'b1                  ),

    .local_mac       (LOCAL_MAC             ),
    .local_ip        (LOCAL_IP              ),
    .local_port      (LOCAL_PORT            ),

    .clk125M_o       (eth_rx_clk            ),
    .fifo_full       (fifo_full             ),
    .fifo_wr         (fifo_wr               ),
    .fifo_din        (fifo_din              ),

    .exter_mac       (                      ),
    .exter_ip        (                      ),
    .exter_port      (                      ),
    .rx_data_length  (                      ),

    .one_pkt_done    (one_pkt_done          ),
    .pkt_error       (pkt_error             ),
    .debug_cal_crc   (                      ),
    .debug_gmii_rxd  (                      ),
    .debug_gmii_rxdv (                      ), 

    .rgmii_rx_clk    (rgmii_rx_clk          ),
    .rgmii_rxdv      (rgmii_rxdv_idly       ),
    .rgmii_rxd       (rgmii_rxd_idly        )
  );

  eth_rx_ctrl
  #(
    .O_DATA_WIDTH (16  )
  )eth_rx_ctrl
  (
    .clk          (eth_rx_clk   ),
    .reset_n      (g_rst_n      ),

    .fifo_full    (fifo_full    ),
    .fifo_wr      (fifo_wr      ),
    .fifo_din     (fifo_din     ),

    .one_pkt_done (one_pkt_done ),
    .pkt_error    (pkt_error    ),

    .rx_data      (wrfifo_din   ),
    .rx_data_valid(wrfifo_wren  )  
  );

  tft_ctrl tft_ctrl
  (
    .clk_ctrl      (tft_ctrl_clk  ),
    .reset_n       (g_rst_n       ),

    .disp_data     (rdfifo_dout   ),
    .disp_data_req (rdfifo_rden   ),
    .visible_hcount(              ),
    .visible_vcount(              ),
    .frame_begin   (frame_begin   ),
    .frame_end     (              ),

    .TFT_rgb       (TFT_rgb       ),
    .TFT_hs        (TFT_hs        ),
    .TFT_vs        (TFT_vs        ),
    .TFT_clk       (TFT_clk       ),
    .TFT_de        (TFT_de        ),
    .TFT_pwm       (TFT_pwm       )
  );

  assign wrfifo_clr = ui_clk_sync_rst || (~pll_locked);
  assign rdfifo_clr = frame_begin || ui_clk_sync_rst;

  wr_ddr3_fifo wr_ddr3_fifo
  (
    .rst           (wrfifo_clr         ), // input  wire rst
    .wr_clk        (eth_rx_clk         ), // input  wire wr_clk
    .rd_clk        (ui_clk             ), // input  wire rd_clk
    .din           (wrfifo_din         ), // input  wire [15 : 0] din
    .wr_en         (wrfifo_wren        ), // input  wire wr_en
    .rd_en         (wrfifo_rden        ), // input  wire rd_en
    .dout          (wrfifo_dout        ), // output wire [127 : 0] dout
    .full          (                   ), // output wire full
    .empty         (wrfifo_empty       ), // output wire empty
    .rd_data_count (wrfifo_rd_cnt      ), // output wire [5 : 0] rd_data_count
    .wr_data_count (                   ), // output wire [8 : 0] wr_data_count
    .wr_rst_busy   (wrfifo_wr_rst_busy ), // output wire wr_rst_busy
    .rd_rst_busy   (wrfifo_rd_rst_busy )  // output wire rd_rst_busy
  );

  rd_ddr3_fifo rd_ddr3_fifo
  (
    .rst           (rdfifo_clr         ), // input wire rst
    .wr_clk        (ui_clk             ), // input wire wr_clk
    .rd_clk        (tft_ctrl_clk       ), // input wire rd_clk
    .din           (rdfifo_din         ), // input wire [127 : 0] din
    .wr_en         (rdfifo_wren        ), // input wire wr_en
    .rd_en         (rdfifo_rden        ), // input wire rd_en
    .dout          (rdfifo_dout        ), // output wire [15 : 0] dout
    .full          (rdfifo_full        ), // output wire full
    .empty         (                   ), // output wire empty
    .rd_data_count (                   ), // output wire [8 : 0] rd_data_count
    .wr_data_count (rdfifo_wr_cnt      ), // output wire [5 : 0] wr_data_count
    .wr_rst_busy   (rdfifo_wr_rst_busy ), // output wire wr_rst_busy
    .rd_rst_busy   (rdfifo_rd_rst_busy )  // output wire rd_rst_busy
  );

//跨时钟域同步操作
  always@(posedge ui_clk)
  begin
    wrfifo_clr_sync_ui_clk <= wrfifo_clr;
    wr_addr_clr <= wrfifo_clr_sync_ui_clk;
  end
  
  always@(posedge ui_clk)
  begin
    rdfifo_clr_sync_ui_clk <= rdfifo_clr;
    rd_addr_clr <= rdfifo_clr_sync_ui_clk;
  end

  always@(posedge ui_clk)
  begin
    wr_fifo_rst_busy_pre <= (wrfifo_wr_rst_busy | wrfifo_rd_rst_busy);
    wr_fifo_rst_busy <= wr_fifo_rst_busy_pre;
  end

  always@(posedge ui_clk)
  begin
    rd_fifo_rst_busy_pre <= (rdfifo_wr_rst_busy | rdfifo_rd_rst_busy);
    rd_fifo_rst_busy <= rd_fifo_rst_busy_pre;
  end

  fifo2mig_axi
  #(
    .WR_DDR_ADDR_BEGIN (0         ),
    .WR_DDR_ADDR_END   (IMAGE_WIDTH*IMAGE_HEIGHT*PIXEL_BYTE ),
    .RD_DDR_ADDR_BEGIN (0         ),
    .RD_DDR_ADDR_END   (IMAGE_WIDTH*IMAGE_HEIGHT*PIXEL_BYTE ),
    
    .AXI_ID            (4'b0000   ),
    .AXI_LEN           (8'd31     )  //axi burst length = 32
  )fifo2mig_axi
  (
    //FIFO Interface ports
    .wr_addr_clr         (wr_addr_clr         ), //1:clear sync ui_clk
    .wr_fifo_rdreq       (wrfifo_rden         ),
    .wr_fifo_rddata      (wrfifo_dout         ),
    .wr_fifo_empty       (wrfifo_empty        ),
    .wr_fifo_rd_cnt      (wrfifo_rd_cnt       ),
    .wr_fifo_rst_busy    (wr_fifo_rst_busy    ),
    
    .rd_addr_clr         (rd_addr_clr         ), //1:clear sync ui_clk
    .rd_fifo_wrreq       (rdfifo_wren         ),
    .rd_fifo_wrdata      (rdfifo_din          ),
    .rd_fifo_alfull      (rdfifo_full         ),
    .rd_fifo_wr_cnt      (rdfifo_wr_cnt       ),
    .rd_fifo_rst_busy    (rd_fifo_rst_busy    ),    
    // Application interface ports
    .ui_clk              (ui_clk              ),
    .ui_clk_sync_rst     (ui_clk_sync_rst     ),
    .mmcm_locked         (mmcm_locked         ),
    .init_calib_complete (init_calib_complete ),
    // Slave Interface Write Address Ports
    .m_axi_awid          (s_axi_awid          ),
    .m_axi_awaddr        (s_axi_awaddr        ),
    .m_axi_awlen         (s_axi_awlen         ),
    .m_axi_awsize        (s_axi_awsize        ),
    .m_axi_awburst       (s_axi_awburst       ),
    .m_axi_awlock        (s_axi_awlock        ),
    .m_axi_awcache       (s_axi_awcache       ),
    .m_axi_awprot        (s_axi_awprot        ),
    .m_axi_awqos         (s_axi_awqos         ),
    .m_axi_awvalid       (s_axi_awvalid       ),
    .m_axi_awready       (s_axi_awready       ),
    // Slave Interface Write Data Ports
    .m_axi_wdata         (s_axi_wdata         ),
    .m_axi_wstrb         (s_axi_wstrb         ),
    .m_axi_wlast         (s_axi_wlast         ),
    .m_axi_wvalid        (s_axi_wvalid        ),
    .m_axi_wready        (s_axi_wready        ),
    // Slave Interface Write Response Ports
    .m_axi_bid           (s_axi_bid           ),
    .m_axi_bresp         (s_axi_bresp         ),
    .m_axi_bvalid        (s_axi_bvalid        ),
    .m_axi_bready        (s_axi_bready        ),
    // Slave Interface Read Address Ports
    .m_axi_arid          (s_axi_arid          ),
    .m_axi_araddr        (s_axi_araddr        ),
    .m_axi_arlen         (s_axi_arlen         ),
    .m_axi_arsize        (s_axi_arsize        ),
    .m_axi_arburst       (s_axi_arburst       ),
    .m_axi_arlock        (s_axi_arlock        ),
    .m_axi_arcache       (s_axi_arcache       ),
    .m_axi_arprot        (s_axi_arprot        ),
    .m_axi_arqos         (s_axi_arqos         ),
    .m_axi_arvalid       (s_axi_arvalid       ),
    .m_axi_arready       (s_axi_arready       ),
    // Slave Interface Read Data Ports
    .m_axi_rid           (s_axi_rid           ),
    .m_axi_rdata         (s_axi_rdata         ),
    .m_axi_rresp         (s_axi_rresp         ),
    .m_axi_rlast         (s_axi_rlast         ),
    .m_axi_rvalid        (s_axi_rvalid        ),
    .m_axi_rready        (s_axi_rready        )
  );

  mig_7series_0 u_mig_7series_0 (
    // Memory interface ports
    .ddr3_addr            (ddr3_addr           ),  // output [13:0]		ddr3_addr
    .ddr3_ba              (ddr3_ba             ),  // output [2:0]		ddr3_ba
    .ddr3_cas_n           (ddr3_cas_n          ),  // output			    ddr3_cas_n
    .ddr3_ck_n            (ddr3_ck_n           ),  // output [0:0]		ddr3_ck_n
    .ddr3_ck_p            (ddr3_ck_p           ),  // output [0:0]		ddr3_ck_p
    .ddr3_cke             (ddr3_cke            ),  // output [0:0]		ddr3_cke
    .ddr3_ras_n           (ddr3_ras_n          ),  // output			    ddr3_ras_n
    .ddr3_reset_n         (ddr3_reset_n        ),  // output			    ddr3_reset_n
    .ddr3_we_n            (ddr3_we_n           ),  // output			    ddr3_we_n
    .ddr3_dq              (ddr3_dq             ),  // inout [15:0]		ddr3_dq
    .ddr3_dqs_n           (ddr3_dqs_n          ),  // inout [1:0]		  ddr3_dqs_n
    .ddr3_dqs_p           (ddr3_dqs_p          ),  // inout [1:0]		  ddr3_dqs_p
    .init_calib_complete  (init_calib_complete ),  // output			    init_calib_complete
    .ddr3_cs_n            (ddr3_cs_n           ),  // output [0:0]		ddr3_cs_n
    .ddr3_dm              (ddr3_dm             ),  // output [1:0]		ddr3_dm
    .ddr3_odt             (ddr3_odt            ),  // output [0:0]		ddr3_odt
    // Application interface ports
    .ui_clk               (ui_clk              ),  // output			    ui_clk
    .ui_clk_sync_rst      (ui_clk_sync_rst     ),  // output			    ui_clk_sync_rst
    .mmcm_locked          (mmcm_locked         ),  // output			    mmcm_locked
    .aresetn              (aresetn             ),  // input			      aresetn
    .app_sr_req           (1'b0                ),  // input			      app_sr_req
    .app_ref_req          (1'b0                ),  // input			      app_ref_req
    .app_zq_req           (1'b0                ),  // input			      app_zq_req
    .app_sr_active        (                    ),  // output			    app_sr_active
    .app_ref_ack          (                    ),  // output			    app_ref_ack
    .app_zq_ack           (                    ),  // output			    app_zq_ack
    // Slave Interface Write Address Ports               
    .s_axi_awid           (s_axi_awid          ),  // input [3:0]			s_axi_awid
    .s_axi_awaddr         (s_axi_awaddr        ),  // input [27:0]	  s_axi_awaddr
    .s_axi_awlen          (s_axi_awlen         ),  // input [7:0]			s_axi_awlen
    .s_axi_awsize         (s_axi_awsize        ),  // input [2:0]			s_axi_awsize
    .s_axi_awburst        (s_axi_awburst       ),  // input [1:0]			s_axi_awburst
    .s_axi_awlock         (s_axi_awlock        ),  // input [0:0]			s_axi_awlock
    .s_axi_awcache        (s_axi_awcache       ),  // input [3:0]			s_axi_awcache
    .s_axi_awprot         (s_axi_awprot        ),  // input [2:0]			s_axi_awprot
    .s_axi_awqos          (s_axi_awqos         ),  // input [3:0]			s_axi_awqos
    .s_axi_awvalid        (s_axi_awvalid       ),  // input			      s_axi_awvalid
    .s_axi_awready        (s_axi_awready       ),  // output			    s_axi_awready
    // Slave Interface Write Data Ports
    .s_axi_wdata          (s_axi_wdata         ),  // input [127:0]	  s_axi_wdata
    .s_axi_wstrb          (s_axi_wstrb         ),  // input [15:0]	  s_axi_wstrb
    .s_axi_wlast          (s_axi_wlast         ),  // input			      s_axi_wlast
    .s_axi_wvalid         (s_axi_wvalid        ),  // input			      s_axi_wvalid
    .s_axi_wready         (s_axi_wready        ),  // output			    s_axi_wready
    // Slave Interface Write Response Ports              
    .s_axi_bid            (s_axi_bid           ),  // output [3:0]	  s_axi_bid
    .s_axi_bresp          (s_axi_bresp         ),  // output [1:0]	  s_axi_bresp
    .s_axi_bvalid         (s_axi_bvalid        ),  // output			    s_axi_bvalid
    .s_axi_bready         (s_axi_bready        ),  // input			      s_axi_bready
    // Slave Interface Read Address Ports                
    .s_axi_arid           (s_axi_arid          ),  // input [3:0]			s_axi_arid
    .s_axi_araddr         (s_axi_araddr        ),  // input [27:0]	  s_axi_araddr
    .s_axi_arlen          (s_axi_arlen         ),  // input [7:0]			s_axi_arlen
    .s_axi_arsize         (s_axi_arsize        ),  // input [2:0]			s_axi_arsize
    .s_axi_arburst        (s_axi_arburst       ),  // input [1:0]			s_axi_arburst
    .s_axi_arlock         (s_axi_arlock        ),  // input [0:0]			s_axi_arlock
    .s_axi_arcache        (s_axi_arcache       ),  // input [3:0]			s_axi_arcache
    .s_axi_arprot         (s_axi_arprot        ),  // input [2:0]			s_axi_arprot
    .s_axi_arqos          (s_axi_arqos         ),  // input [3:0]			s_axi_arqos
    .s_axi_arvalid        (s_axi_arvalid       ),  // input			      s_axi_arvalid
    .s_axi_arready        (s_axi_arready       ),  // output			    s_axi_arready
    // Slave Interface Read Data Ports                   
    .s_axi_rid            (s_axi_rid           ),  // output [3:0]	  s_axi_rid
    .s_axi_rdata          (s_axi_rdata         ),  // output [127:0]  s_axi_rdata
    .s_axi_rresp          (s_axi_rresp         ),  // output [1:0]	  s_axi_rresp
    .s_axi_rlast          (s_axi_rlast         ),  // output			    s_axi_rlast
    .s_axi_rvalid         (s_axi_rvalid        ),  // output			    s_axi_rvalid
    .s_axi_rready         (s_axi_rready        ),  // input			      s_axi_rready
    // System Clock Ports                                
    .sys_clk_i            (mig_clk200M         ),
    .sys_rst              (mig_reset_n         )   // input sys_rst
  );

endmodule